# -*- coding: utf-8 -*-
# Copyright (c) 2010 Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok
from silva.core.messages.interfaces import IMessageService, IMessage
from silva.core.cache.store import SessionStore


class Message(object):
    grok.implements(IMessage)

    namespace = None

    def __init__(self, string, namespace='message'):
        self.content = string
        self.namespace = namespace

    def __str__(self):
        return self.content

    def __unicode__(self):
        return self.content


STORE_KEY = 'messages'


class MessageService(grok.GlobalUtility):
    grok.provides(IMessageService)
    grok.implements(IMessageService)


    def __retrieve(self, request):
        store = SessionStore(request)
        messages = store.get(STORE_KEY, list())
        return (store, messages)

    def send(self, message_str, request, namespace=u"message"):
        store, messages = self.__retrieve(request)
        messages.append(Message(message_str, namespace=namespace))
        store.set(STORE_KEY, messages)

    def receive(self, request, namespace=u"message"):
        store, messages = self.__retrieve(request)
        keep = list()
        reception = list()
        for message in messages:
            if message.namespace != namespace:
                keep.append(message)
            else:
                reception.append(message)
        store.set(STORE_KEY, keep)
        return reception

    def receive_all(self, request):
        store, messages = self.__retrieve(request)
        store.set(STORE_KEY, list())
        return messages



from zope.interface import Interface, Attribute


class IMessage(Interface):
    namespace = Attribute('message namespace')

    def __str__():
        """ Render message as text
        """

    def __unicode__():
        """Render message as unicode
        """


class IMessageService(Interface):
    """ Utility for registering messages it should receive only IMessages
    """
    def send(message, request, namespace=u"message"):
        """ send message
        """

    def receive(request, namespace=u"message"):
        """ receive message, return a list of tuples (type, message)
        """



# -*- coding: utf-8 -*-
# Copyright (c) 2010 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from Products.Silva.testing import FunctionalLayer
from zope.component import getUtility
from zope.interface.verify import verifyObject
from zope.publisher.browser import TestRequest

from silva.core.messages.interfaces import IMessageService


def messages_to_string(messages):
    return [(message.namespace, str(message),) for message in messages]


class UtilityTest(unittest.TestCase):
    # XXX I don't think this test needs FunctionalLayer
    layer = FunctionalLayer

    def setUp(self):
        self.request = TestRequest()

    def test_utility(self):
        service = getUtility(IMessageService)
        self.assertTrue(verifyObject(IMessageService, service))

        service.send(u"some unicode message", self.request)
        self.assertEquals(
            [(u'message', u"some unicode message",)],
            messages_to_string(service.receive_all(self.request)))


class MessagingTest(unittest.TestCase):

    layer = FunctionalLayer

    def setUp(self):
        self.request = TestRequest()
        self.service = getUtility(IMessageService)

    def test_no_messages(self):
        self.assertEquals([], self.service.receive(self.request))

    def test_multiple_messages(self):
        self.service.send(u'message1', self.request)
        self.service.send(u'message2', self.request)
        self.assertEquals(
            [(u'message', u'message1',),
             (u'message', u'message2',)],
            messages_to_string(self.service.receive_all(self.request)))
        self.assertEquals([], self.service.receive_all(self.request))

    def test_multiple_messages_of_different_type(self):
        self.service.send(u'message_type1', self.request, namespace=u"type1")
        self.service.send(u'message_type2', self.request, namespace=u"type2")
        self.service.send(u'message_type1', self.request, namespace=u"type1")

        def receive(type):
            return self.service.receive(self.request, type)

        self.assertEquals(
            [u'message_type1',
             u'message_type1',],
            map(str, receive(u'type1')))
        self.assertEquals([], receive(u'type1'))

        self.assertEquals(
            [u'message_type2',], map(str, receive(u'type2')))
        self.assertEquals([], receive(u'type2'))

    def test_multiple_messages_of_different_type_fetch_all_at_once(self):
        self.service.send(u'message_type1', self.request, namespace=u"type1")
        self.service.send(u'message_type2', self.request, namespace=u"type2")
        self.service.send(u'message_type1', self.request, namespace=u"type1")

        self.assertEquals(
            [(u'type1', u'message_type1',),
             (u'type2', u'message_type2',),
             (u'type1', u'message_type1',)],
            messages_to_string(self.service.receive_all(self.request)))
        self.assertEquals([], self.service.receive_all(self.request))


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(UtilityTest))
    suite.addTest(unittest.makeSuite(MessagingTest))
    return suite



from setuptools import setup, find_packages
import os

version = '2.3'

tests_require = [
    'Products.Silva [test]',
    'zope.publisher',
    'zope.component',]

setup(name='silva.core.messages',
      version=version,
      description="User feedback mechanism for Silva CMS",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      classifiers=[
        "Framework :: Zope2",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: BSD License",
        ],
      keywords='silva core messages',
      author='Infrae',
      author_email='info@infrae.com',
      url='https://github.com/silvacms/silva.core.messages',
      license='BSD',
      package_dir={'': 'src'},
      packages=find_packages('src'),
      namespace_packages=['silva', 'silva.core'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
        'five.grok',
        'silva.core.cache',
        'zope.interface',
      ],
      tests_require=tests_require,
      extras_require={'test': tests_require},
      )
